/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.io.IOException;
import java.text.ParseException;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import cx.ath.matthew.cgi.CGI;
import cx.ath.matthew.cgi.CGIHeaderSentException;
import cx.ath.matthew.cgi.CheckBox;
import cx.ath.matthew.cgi.DisplayField;
import cx.ath.matthew.cgi.HiddenField;


import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.StateBuilderException;

public class CGIAddParticipant implements CGICommand
{
   private void addParticipant(CGI cgi, Map plugins, Map params) throws AssassinSQLException, IOException, ParseException, StateBuilderException
   {
      if ("2".equals(params.get("pass"))) {
         HTMLForm form = new HTMLForm("umpire.cgi?add-participant", "Add");
         Assassin a = (Assassin) params.get("assassin");
         Event e = (Event) params.get("event");
         if (Debug.debug) Debug.print(umpirecgi.class, "assassin: "+a+", event: "+e);
         LightweightStateBuilder lsb = new LightweightStateBuilder(Tools.getNowTime());
         lsb.registerInterest(a);
         Map myState = null;
         try {
            myState = lsb.generateState();
         } catch (StateBuilderException sbe) {
            cgi.out("<div class=\"errormessage\">");
            cgi.out("  <p><b>Internal Error</b></p>");
            cgi.out("  <p>State Building failed! Very bad!!</p>");
            cgi.out("</div>");
            cgi.flush();
            throw new RuntimeException();
         }
         cgi.out("<h2>Add Participant to Event</h2>");
         cgi.out("<p>");
         cgi.out("  Select Pseudonym for Participant:");
         cgi.out("</p>");
         form.addField(new HiddenField("pass","3"));
         form.addField(new HiddenField("assassin",""+a));
         form.addField(new HiddenField("event",""+e));
         if (Debug.debug) Debug.print(umpirecgi.class, "state: "+myState.get(a));
         if (Debug.debug) Debug.print(umpirecgi.class, "pseudonyms: "+((AssassinState) myState.get(a)).getPseudonyms());
         form.addField(new CanonicalEntityField("pseudonym","Pseudonym:",((AssassinState) myState.get(a)).getPseudonyms(),(CanonicalEntity) null));
         form.addField(new CheckBox("nopseud", "Use Real Name", false));

         umpirecgi.currentAuth.clearSessionQuestions();
         Map statev = Tools.viewerifyStateMap(myState);
         EventViewer ev = (EventViewer) e.getViewer();
         Iterator plgs = plugins.values().iterator();
         while (plgs.hasNext()) {
           Plugin pl = (Plugin) plgs.next();
           AssassinViewer av = (AssassinViewer) a.getViewer();
           Vector qs = pl.participantAdded(ev, av, statev, null);
           Iterator qi = qs.iterator();
           for (int i=0; qi.hasNext(); i++) {
              Question q = (Question) qi.next();
               form.addField(CGITools.getPluginField(pl, q, i));
               umpirecgi.currentAuth.addSessionQuestion(pl, q);
            }
         }
         Event.commitAll();
         StateChange.commitAll();
         form.display(cgi);
      } else if ("3".equals(params.get("pass"))) {
         // Done asking questions, so update event
         cgi.out("<h2>Add Participant to Event</h2>");
         cgi.out("<div class=\"donemessage\">");
         cgi.out("  <p><b>Participant Successfully Added</b></p>");
         cgi.out("  <p>&nbsp;</p>");
         cgi.out("</div>");
         Pseudonym pseudonym = (Pseudonym) params.get("pseudonym");
         Assassin a = (Assassin) params.get("assassin");
         Event e = (Event) params.get("event");
         
         LightweightStateBuilder lsb = new LightweightStateBuilder(Tools.getNowTime());
         lsb.registerInterest(a);
         Map myState = null;
         try {
            myState = lsb.generateState();
         } catch (StateBuilderException sbe) {
            cgi.out("<h2>State Building failed! Very bad!!</h2>");
            cgi.flush();
            throw new RuntimeException();
         }
         Map statev = Tools.viewerifyStateMap(myState);
         AssassinViewer av = (AssassinViewer) a.getViewer();
         EventViewer ev = (EventViewer) ((Event) params.get("event")).getViewer();

         // parse out plugin fields from POST
         Iterator plgs = plugins.values().iterator();
         while (plgs.hasNext()) {
            Plugin pl = (Plugin) plgs.next();
            Vector qs = CGITools.parseQuestions(pl, params);
            pl.participantAdded(ev, av, statev, qs);
         }
         Event.commitAll();
         StateChange.commitAll();
         umpirecgi.currentAuth.clearSessionQuestions();

         try { cgi.header("Refresh", "2; URL=umpire.cgi?edit-event,"+e.getEid()); } catch (CGIHeaderSentException CGIHSe) {} 
         int m = e.getMaxParticipantID();
         if ("on".equals(params.get("nopseud")))
            e.setParticipant(++m, a, null);
         else
            e.setParticipant(++m, a, pseudonym);
         Event.commitAll();
      } else {
         HTMLForm form = new HTMLForm("umpire.cgi?add-participant", "Add");
         cgi.out("<h2>Add a participant in an Event</h2>");
         cgi.out("<p>");
         cgi.out("  Select Event to add participant to and assassin to add:");
         cgi.out("</p>");
         if (null != params.get("1") && !"".equals(params.get("1"))) {
            Event e = Event.getByID(Integer.parseInt((String) params.get("1")));
            form.addField(new DisplayField("Event:", e.getPrintable()));
            form.addField(new HiddenField("event",""+e));
         }
         else {
            List l = Event.getAllEventsWithHeadline();
            Event.preCacheSubClasses(Event.READPARTICIPANTS);
            form.addField(new CanonicalEntityField("event","Event:",l,(CanonicalEntity) null));
         }
         form.addField(new CanonicalEntityField("assassin","Participant:",Assassin.getAllAssassinsOrderByRealName(),(CanonicalEntity)null));
         form.addField(new HiddenField("pass","2"));
         form.display(cgi);
      }
   }

   public void command(CGI cgi, Map plugins, Map params, long nowtime) 
      throws AssassinSQLException, IOException, ParseException, StateBuilderException { addParticipant(cgi, plugins, params); } 
   public String getName() { return "Add Participant"; }
   public String getDescription() { return "Adds an assassin as a participant in an existing event"; }
   public AuthLevel accessLevel () { return AuthLevel.PLAYER; }
   public String getSection() { return "Event"; }
   public boolean isVisible() {	return true; }

}

