/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.io.IOException;
import java.text.ParseException;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import cx.ath.matthew.cgi.CGI;
import cx.ath.matthew.cgi.CGIHeaderSentException;
import cx.ath.matthew.cgi.CheckBox;
import cx.ath.matthew.cgi.DisplayField;
import cx.ath.matthew.cgi.HiddenField;
import cx.ath.matthew.cgi.TextArea;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.StateBuilderException;

public class CGIAddReport implements CGICommand
{
   void addReport(CGI cgi, Event e, Map plugins, Map params) throws AssassinSQLException, IOException , StateBuilderException
   {
      if ("2".equals(params.get("pass"))) {
         HTMLForm form = new HTMLForm("umpire.cgi?add-report", "Add");
         Assassin a = (Assassin) params.get("assassin");
         e = (Event) params.get("event");
         String comment = (String) params.get("comment");
         if (Debug.debug) Debug.print(umpirecgi.class, "assassin: "+a+", event: "+e);
         if (!"on".equals(comment)) {
            LightweightStateBuilder lsb = new LightweightStateBuilder(Tools.getNowTime());
            lsb.registerInterest(a);
            Map myState = null;
            try {
               myState = lsb.generateState();
            } catch (StateBuilderException sbe) {
               cgi.out("<div class=\"errormessage\">");
               cgi.out("  <p><b>Internal Error</b></p>");
               cgi.out("  <p>State Building failed! Very bad!!</p>");
               cgi.out("</div>");
               cgi.flush();
               throw new RuntimeException();
            }
            cgi.out("<h2>Add Report</h2>");
            cgi.out("<p>");
            cgi.out("  Select Pseudonym for Assassin:");
            cgi.out("</p>");
            form.addField(new HiddenField("assassin",""+a));
            if (Debug.debug) Debug.print(umpirecgi.class, "state: "+myState.get(a));
            if (Debug.debug) Debug.print(umpirecgi.class, "pseudonyms: "+((AssassinState) myState.get(a)).getPseudonyms());
            form.addField(new CanonicalEntityField("pseudonym","",((AssassinState) myState.get(a)).getPseudonyms(),(CanonicalEntity) null));
            form.addField(new CheckBox("nopseud", "Use Real Name", false));
         }
         else {
            cgi.out("<h2>Add Report</h2>");
            cgi.out("<p>");
            cgi.out("  Adding Comment:");
            cgi.out("</p>");
         }
         cgi.out("<p>");
         cgi.out("   You can include HTML in your report, but please remember");
         cgi.out("   to match all opening tags with closing tags, and escape");
         cgi.out("   &amp; with &amp;amp; and &gt;/&lt; with &amp;gt;/&amp;lt;");
         cgi.out("   if they are not in tags. Also, remember that all non-ascii");
         cgi.out("   characters (accents, etc) should be entered in the UTF-8");
         cgi.out("   encoding. HTML Entities (eg &amp;eacute;) can be entered as");
         cgi.out("   &lt;entity name='eacute'/&gt;.");
         cgi.out("   Reports may also contain the special sequences");
         cgi.out("   [Pn], [Nn] and [Ln],");
         cgi.out("   where n is the participant number.");
         cgi.out("   [Pn] will be replaced by their pseudonym,");
         cgi.out("   [Nn] by their name");
         cgi.out("   and [Ln] by the list of all their pseudonyms.");
         cgi.out("</p>");
         form.addField(new HiddenField("pass","3"));
         form.addField(new HiddenField("event",""+e));
         form.addField(new TextArea("report","Report:","Report Text goes here", 80, 25));
         form.display(cgi);
      } else if ("3".equals(params.get("pass"))) {
         // Done asking questions, so update event
         cgi.out("<h2>Add Report</h2>");
         cgi.out("<div class=\"donemessage\">");
         cgi.out("  <p><b>Report/Comment Successfully Added</b></p>");
         cgi.out("  <p>&nbsp;</p>");
         cgi.out("</div>");
         Pseudonym pseudonym = (Pseudonym) params.get("pseudonym");
         Assassin a = (Assassin) params.get("assassin");
         if (!umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE))
            if (null == a || 
                  !a.getPlayer().equals(umpirecgi.currentAuth.getPlayerViewer().getEntity())) {
               cgi.out("<div class=\"errormessage\">");
               cgi.out("   <p><b>Insufficient Privileges</b></p>");
               cgi.out("   <p>Only the umpire can add comments or reports from other people.</p>");
               cgi.out("</div>");
               return;
            }
         String report = (String) params.get("report");
         e = (Event) params.get("event");
         Report r = new Report();
         r.setAssassin(a);
         if ("on".equals(params.get("nopseud")))
            r.setPseudonym(null);
         else
            r.setPseudonym(pseudonym);
         r.setText(report);
         e.addReport(r);
         Event.commitAll();
         try { cgi.header("Refresh", "2; URL=umpire.cgi?edit-event,"+e.getEid()); } catch (CGIHeaderSentException CGIHSe) {} 
         CGIGeneratePages.generatePages(cgi, plugins, params, false);
      } else {
         HTMLForm form = new HTMLForm("umpire.cgi?add-report", "Add");
         cgi.out("<h2>Add Report</h2>");
         cgi.out("<p>");
         cgi.out("  Select Event to add report to and reporting assassin:");
         cgi.out("</p>");
         if (null == e && null != params.get("1") && !"".equals(params.get("1"))) {
            e = Event.getByID(Integer.parseInt((String) params.get("1")));
            form.addField(new DisplayField("Event:", e.getPrintable()));
            form.addField(new HiddenField("event",""+e));
            if (umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE))
               form.addField(
                     new CanonicalEntityField(
                        "assassin",
                        "Assassin:",
                        new Vector(e.getParticipants().values()),
                        (CanonicalEntity)null
                        ));
            else
               form.addField(
                     new CanonicalEntityField(
                        "assassin",
                        "Assassin:",
                        umpirecgi.currentAuth.getPlayerViewer().getAssassins(),
                        (CanonicalEntity)null
                        ));

         }
         else {
            List l = Event.getAllEventsWithHeadline();
            if (Debug.debug) Debug.print(umpirecgi.class, "Got events");
            Event.preCacheSubClasses(Event.READPARTICIPANTS);
            if (Debug.debug) Debug.print(umpirecgi.class, "Got subclasses");
            form.addField(new CanonicalEntityField("event","Event:",l,e));
            if (Debug.debug) Debug.print(umpirecgi.class, "Field added");
            if (umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE))
               form.addField(
                     new CanonicalEntityField(
                        "assassin",
                        "Assassin:",
                        Assassin.getAllAssassinsOrderByRealName(),
                        (CanonicalEntity)null
                        ));
            else
               form.addField(
                     new CanonicalEntityField(
                        "assassin",
                        "Assassin:",
                        umpirecgi.currentAuth.getPlayerViewer().getAssassins(),
                        (CanonicalEntity)null
                        ));
            if (Debug.debug) Debug.print(umpirecgi.class, "Got all assassins");

         }
         if (umpirecgi.currentAuth.isAtLeast(AuthLevel.UMPIRE))
            form.addField(new CheckBox("comment","Add Comment:",false));
         form.addField(new HiddenField("pass","2"));
         form.display(cgi);
         if (Debug.debug) Debug.print(umpirecgi.class, "Form displayed");
      }
   }

   public void command(CGI cgi, Map plugins, Map params, long nowtime) 
      throws AssassinSQLException, IOException, ParseException, StateBuilderException {  addReport(cgi, null, plugins, params); } 
   public String getName() { return "Add Report"; }
   public String getDescription() { return "Adds a report to an event"; }
   public AuthLevel accessLevel () { return AuthLevel.PLAYER; }
   public String getSection() { return "Event"; }
   public boolean isVisible() {	return true; }
}

