/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import cx.ath.matthew.cgi.CGI;
import cx.ath.matthew.cgi.CheckBox;
import cx.ath.matthew.cgi.DisplayField;
import cx.ath.matthew.cgi.DropDown;
import cx.ath.matthew.cgi.Field;
import cx.ath.matthew.cgi.HiddenField;
import cx.ath.matthew.cgi.TextField;


import org.ucam.srcf.assassins.exceptions.NotSerialisedEntityException;

/**
 * Contains lots of static methods used by the CGI
 */
public abstract class CGITools
{
   public static void posttoform(Map POST, HTMLForm form, String[] omit)
   {
      Arrays.sort(omit);
      Iterator i = POST.keySet().iterator();
      while (i.hasNext()) {
         String key = (String) i.next();
         if ("authuser".equals(key)) continue;
         if ("authpw".equals(key)) continue;
         if ("submit".equals(key)) continue;
         if (Arrays.binarySearch(omit, key) < 0) {
            Object o =  POST.get(key);
            if (o instanceof List) {
               Iterator j = ((List) o).iterator();
               while (j.hasNext()) 
                  form.addField(new HiddenField(key, j.next().toString()));
            } else
               form.addField(new HiddenField(key, POST.get(key).toString()));
         }
      }
   }

   static Map translateEntities(Map input)
   {
      Map output = new HashMap();
      Iterator i = input.keySet().iterator();
      while (i.hasNext()) {
         String k = (String) i.next();
         Object o = input.get(k);
         if (o instanceof List) {
            List l = new LinkedList();
            Iterator j = ((List) o).iterator();
            while (j.hasNext()) {
               String s = (String) j.next();
               Object p = null;
               try {
                  p = CanonicalEntity.deSerialise(s);
               } catch (NotSerialisedEntityException NSEe) {
                  p = s;
               }
               l.add(p);
            }
            output.put(k, l);
         } 
         else if (o instanceof String) {
            String s = (String) o;
            Object p = null;
            try {
               p = CanonicalEntity.deSerialise(s);
            } catch (NotSerialisedEntityException NSEe) {
               p = s;
            }
            output.put(k, p);
         }
      }
      return output;
   }
   
   static Map viewerifyParamMap(Map in)
   {
      Map out = new HashMap();
      Iterator i = in.keySet().iterator();
      while (i.hasNext())
      {
         String key = (String) i.next();
         Object value = in.get(key);
         if (value instanceof String)
            out.put(key, value);
         else if (value instanceof Entity)
            out.put(key, ((Entity) value).getViewer());
         else if (value instanceof List) {
            List l = new LinkedList();
            Iterator j = ((List) value).iterator();
            while (j.hasNext()) {
               Object o = j.next();
               if (o instanceof String)
                  l.add(o);
               else if (o instanceof Entity)
                  l.add(((Entity) o).getViewer());
            }
            out.put(key, l);
         }
      }
      return out;
   }
   
   static Vector parseQuestions(Plugin pl, Map params) {
      return parseQuestions(pl, params, 0);
   }
   
   /**
    * Returns the vector of questions saved for this plugin, with the answers
    * filled out from the POST data
    */
   static Vector parseQuestions(Plugin pl, Map params, int base)
   {
      Vector qs = umpirecgi.currentAuth.getSessionQuestions(pl);
      if (null == qs) return new Vector();
      Iterator qi = qs.iterator();
      for (int i=0; qi.hasNext(); ) {
         Question q = (Question) qi.next();
         if (Debug.debug) Debug.print(Question.class, "Got question: "+q);
         String s = "";
         if (q instanceof TextQuestion) s = "/Text/";
         else if (q instanceof YesNoQuestion) s = "/Check/";
         else if (q instanceof OptionQuestion) s = "/Option/";
         s = "::"+pl.getPluginName()+s+(base+i);
         if (Debug.debug) Debug.print(Question.class, "Got param name: "+s);
         String par = (String)params.get(s);
         if (Debug.debug) Debug.print(Question.class, "Got param result: "+par);
         if (null == par) continue;
         else {
            i++;
            if (q instanceof YesNoQuestion) 
               if ("on".equals(par))
                  ((YesNoQuestion) q).setAnswer(true);
               else
                  ((YesNoQuestion) q).setAnswer(false);
            else if (q instanceof TextQuestion) 
               ((TextQuestion) q).setAnswer(par);
            else if (q instanceof OptionQuestion) {
               ((OptionQuestion) q).setAnswer(Integer.parseInt(par));
               if (Debug.debug) Debug.print(Question.class, "Option set answer: "+((OptionQuestion) q).getAnswerString());
            }
         }
      }
      return qs;
   }

   /**
    * Turn a Question into an HTML Field. 
    * Also includes plugin identification information
    */
   static Field getPluginField(Plugin p, Question q, int index)
   {
      if (q instanceof OptionQuestion) {
         List l = ((OptionQuestion) q).getOptions();
         if ( l.size() > 0 && l.get(0) instanceof CanonicalEntity)
            return new CanonicalEntityField("::"+p.getPluginName()+"/Option/"+index, q.getLabel(), l, (CanonicalEntity) l.get(((OptionQuestion) q).getDefault()), true);
         if ( l.size() > 0 && l.get(0) instanceof CanonicalViewer)
            return new CanonicalEntityField("::"+p.getPluginName()+"/Option/"+index, q.getLabel(), l, (CanonicalViewer) l.get(((OptionQuestion) q).getDefault()), true);
         else
            return new DropDown("::"+p.getPluginName()+"/Option/"+index, q.getLabel(), ((OptionQuestion) q).getOptions(), ((OptionQuestion) q).getDefault(), true);
      }
      else if (q instanceof TextQuestion)
         return new TextField("::"+p.getPluginName()+"/Text/"+index, q.getLabel(), ((TextQuestion) q).getDefault());
      else if (q instanceof DisplayData)
         return new DisplayField("", ((DisplayData) q).getInfo());
      else if (q instanceof YesNoQuestion)
         return new CheckBox("::"+p.getPluginName()+"/Check/"+index, q.getLabel(), ((YesNoQuestion) q).getDefault());
      else return null;
   }

   public static void printlink(CGI cgi, String id, String target, String text)
   {
      printlink(cgi, id, target, text, null);
   }
   public static void printlink(CGI cgi, String id, String target, String text, String cclass)
   {
      if (umpirecgi.customnowtime) {
         String chr;
         if (target.indexOf('?') < 0) chr = "?"; else chr = "&amp;";
         cgi.out("<a "+(null==cclass?"":"class='"+cclass+"'")+
               " href=\""+target+chr+"nowtime="+
               Tools.formatTime(Tools.getNowTime())+"\">"+text+"</a>");
      } else
         cgi.out("<a "+(null==cclass?"":"class='"+cclass+"'")+
               " href=\""+target+"\">"+text+"</a>");
   }

}

