/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins.strawberry;

import java.text.ParseException;

import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;

import java.util.HashMap;
import java.util.Map;
import java.util.Vector;
import java.util.List;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Set;
import java.util.HashSet;

import org.ucam.srcf.assassins.AssassinActivation;
import org.ucam.srcf.assassins.AssassinViewer;
import org.ucam.srcf.assassins.AssassinStateViewer;
import org.ucam.srcf.assassins.ConfigProperty;
import org.ucam.srcf.assassins.EventViewer;
import org.ucam.srcf.assassins.Plugin;
import org.ucam.srcf.assassins.StateChange;
import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.StateBuilderException;
import org.ucam.srcf.assassins.xml.Config;
import org.ucam.srcf.assassins.Assassin;
import org.ucam.srcf.assassins.PluginCommand;
import org.ucam.srcf.assassins.TextQuestion;
import org.ucam.srcf.assassins.Debug;
import org.ucam.srcf.assassins.Death;
import org.ucam.srcf.assassins.Resurrection;
import org.ucam.srcf.assassins.PluginStateChange;
import org.ucam.srcf.assassins.Event;
import org.ucam.srcf.assassins.PlayerViewer;
import org.ucam.srcf.assassins.Timer;
import org.ucam.srcf.assassins.Tools;

import org.ucam.srcf.assassins.exceptions.ConfigFileException;

/**
 * Interface for java classes which act as plug-in game modifiers.
 */
public class StrawberryTools extends Plugin
{
   /**
    * Does the given StateChagne have anything to do with anyone in the set
    * involving?
    * @return true if it does, false otherwise.
    */
   static boolean doesSCInvolveSet(StateChange sc, Set involving) throws AssassinSQLException {
      if(sc instanceof Death && involving.contains(((Death) sc).getKiller()))
         return true;
      if(involving.contains(sc.getAssassinViewer()))
         return true;
      return false;
   }

   /**
    * Does the given event have any particiapnts from the given Set?
    */
   static boolean eventContainsParticipantsFromSet(EventViewer e, Set s) throws AssassinSQLException {
      Iterator i = e.getStateChanges().iterator();
      while(i.hasNext()) {
         StateChange sc = (StateChange) i.next();
         if(sc instanceof Death)
            if(s.contains(((Death) sc).getKiller()))
               return true;
         if(s.contains(sc.getAssassinViewer()))
            return true;
      }
      return false;
   }
}
