/**
 * Class for dealing with static scoring.
 */

package org.ucam.srcf.assassins.strawberry;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.ucam.srcf.assassins.AssassinViewer;
import org.ucam.srcf.assassins.Death;
import org.ucam.srcf.assassins.Debug;
import org.ucam.srcf.assassins.exceptions.AssassinSQLException;

public class TestStatic implements StaticScoringAlgorithm {
   /**
    * Initilise algorithm. 
    * Will be called before any other functions.
    * @param n Maximum number of assassins the algorithm has to deal with
    */
   public void init (int n) {
      scores=new HashMap();
   }

   /** 
    * Scores the deaths the algorithm knows about.
    * This is <i>no longer</i> the final call of the algorithm, and in
    * reality you will get intermixed calls of addDeath and score as
    * we iterate through the game's progress.
    * @return Map from AssassinViewer to Double representing score.
    */
   public Map score() {
      return scores;
   }

   private Map scores;

   /**
    * Tell the algorithm about a death.
    * May <i>no longer</i> assume score() hasn't been called yet.
    * @param d Death we're telling it about.
    */
   public void addDeath(Death d) throws AssassinSQLException {
      double killer = getScore((AssassinViewer) d.getKiller());
      double victim = getScore((AssassinViewer) d.getVictim());
      killer += 100;
      victim += 1;
      Double killerD = new Double(killer);
      Double victimD = new Double(victim);
      scores.put(d.getKiller(), killerD);
      Debug.print(this, "Putting "+d.getKiller()+" ("+d.getKiller().getPlayerViewer().getName()+") with hashcode "+d.getKiller().hashCode()+ " into map with value "+killerD);
      scores.put(d.getVictim(), victimD);
      Debug.print(this, "Putting "+d.getVictim()+" ("+d.getVictim().getPlayerViewer().getName()+") with hashcode "+d.getVictim().hashCode()+ " into map with value "+victimD);
   }

   private void debugPrintMap() throws AssassinSQLException {
      Debug.print(this, "Content of scores map:");
      Iterator i = scores.keySet().iterator();
      while(i.hasNext()) {
         AssassinViewer a = (AssassinViewer) i.next();
         Double score = (Double) scores.get(a);
         Debug.print(this, a+" ("+a.getPlayerViewer().getName()+") -> "+score);
      }
   }

   private double getScore(AssassinViewer a) throws AssassinSQLException {
      Double score = (Double) scores.get(a);
      if(Debug.debug) Debug.print(this, "Retrieving score for "+a+" ("+a.getPlayerViewer().getName()+") hashcode "+a.hashCode()+" who has score "+score);
      double ret = 0;
      if(score != null)
         ret = score.doubleValue();
      return ret;
   }

   /**
    * Tells the algorithm about an innocent death(s).
    * It is up to the alogrithm to use it as it sees fit
    * @param who Who killed it/them?
    * @param count How many they killed.
    */
   public void reportInnocents(AssassinViewer who, int count) throws AssassinSQLException {
   }

   /**
    * Tells the algorithm about bonuses.
    * It is up to the algorithm to actually apply them
    * @param who Who gets the bonus.
    * @param value How much is it worth?
    */
   public void giveBonus(AssassinViewer who, double value) throws AssassinSQLException {
   }
}
